﻿// -----------------------------------------------------------------------
// <copyright file="EnumGenerator.cs" company="none">
// Copyright (c) 2012 M. Alberti, xanatos(at)live.it
// Distributed under the MIT/X11 software license, see the accompanying
// file COPYING or http://www.opensource.org/licenses/mit-license.php.
// </copyright>
// -----------------------------------------------------------------------

namespace FluentSerializer.Generators
{
    using System;
    using System.Linq.Expressions;
    using FluentSerializer;

    /// <summary>
    /// 
    /// </summary>
    public static class EnumGenerator
    {
        /// <summary>
        /// 
        /// </summary>
        /// <param name="serializer"></param>
        /// <param name="parametersType"></param>
        /// <param name="type"></param>
        /// <param name="helper"></param>
        /// <returns></returns>
        public static Descriptor BuildSerializer(Serializer serializer, Type parametersType, Type type, Type helper)
        {
            Type subType = type.GetEnumUnderlyingType();

            Descriptor descriptor1 = Serializer.GetSerializer(serializer, subType, subType, helper);

            ParameterExpression item = Expression.Parameter(type);
            ParameterExpression parameters = null;

            LambdaExpression deserializeExpression;
            LambdaExpression serializeExpression;
            LambdaExpression sizeExpression;

            /***/
            {
                Expression deserializedItem = Serializer.MakeFromLambda(descriptor1.DeserializeExpression, descriptor1.IsUniversal, Serializer.ReaderExpression, descriptor1.DefaultExpression, parameters);
                deserializedItem = Expression.Convert(deserializedItem, type);

                deserializeExpression = Serializer.MakeLambda(deserializedItem, descriptor1.IsUniversal, Serializer.ReaderExpression, item, parameters);
            }

            /***/
            {
                Expression serializedItem = Expression.ConvertChecked(item, subType);

                serializeExpression = Serializer.MakeLambda(Serializer.MakeFromLambda(descriptor1.SerializeExpression, descriptor1.IsUniversal, Serializer.WriterExpression, serializedItem, parameters), descriptor1.IsUniversal, Serializer.WriterExpression, item, parameters);
            }

            /***/
            {
                sizeExpression = Serializer.MakeLambda(Serializer.MakeFromLambda(descriptor1.SizeExpression, descriptor1.IsUniversal, descriptor1.DefaultExpression, parameters), descriptor1.IsUniversal, item, parameters);
            }

            var descriptor2 = new Descriptor(type, descriptor1.IsUniversal, deserializeExpression, DeserializationHandling.RequireAssignment, serializeExpression, sizeExpression);
            return descriptor2;
        }
    }
}